// vip_comm.h

#if !defined(VIP_HEADER)
#define VIP_HEADER

// NOTE: client should NOT define VIP_DLL!!!

#ifdef VIP_DLL
#define LINKAGE __declspec(dllexport)
#else
#define LINKAGE __declspec(dllimport)
#endif

// IO links.
#define VIP_NO_LINK       -1
#define VIP_ETHERNET_LINK 0
#define VIP_SERIAL_LINK   1
#define VIP_COM1 0
#define VIP_COM2 1
#define VIP_COM3 2
#define VIP_COM4 3

#define VIP_DISPL_TST_PTTRN_STRTUP 0
#define VIP_STANDALONE_STRTUP      1
#define VIP_ACTIVE_ACQ_STRTUP      2

// Return Codes
#define VIP_NO_ERR       0x0000
#define VIP_COMM_ERR     0x0001
#define VIP_STATE_ERR    0x0002
#define VIP_DATA_ERR     0x0004
#define VIP_NO_DATA_ERR  0x0008
#define VIP_NO_SRVR_ERR  0x0010
#define VIP_SETUP_ERR    0x0020
#define VIP_NO_CAL_ERR   0x0040
#define VIP_NO_IMAGE_ERR 0x0080
#define VIP_NOT_IMPL_ERR 0x4000
#define VIP_OTHER_ERR    0x8000

// System Version Number types
#define VIP_MOTHERBOARD_VER    0
#define VIP_SYS_SW_VER         1
#define VIP_GLOBAL_CTRL_VER    2
#define VIP_GLOBAL_CTRL_FW_VER 3
#define VIP_RECEPTOR_VER       4
#define VIP_RECEPTOR_FW_VER    5
#define VIP_IPS_VER            6
#define VIP_VIDEO_OUT_VER      7
#define VIP_VIDEO_OUT_FW_VER   8

// Image Types
#define VIP_CURRENT_IMAGE     0
#define VIP_OFFSET_IMAGE      1
#define VIP_GAIN_IMAGE        2
#define VIP_BASE_DEFECT_IMAGE 3
#define VIP_AUX_DEFECT_IMAGE  4
#define VIP_TEST_IMAGE        5

// Software Handshaking Constants
#define VIP_SW_PREPARE           0
#define VIP_SW_VALID_XRAYS       1
#define VIP_SW_RADIATION_WARNING 2
#define VIP_SW_RESET             3

// Rad Scaling Type Constants
#define VIP_RAD_SCALE_NONE 0
#define VIP_RAD_SCALE_UP   1
#define VIP_RAD_SCALE_DOWN 2
#define VIP_RAD_SCALE_BOTH 3

// Other Constants
#define VIP_ACQ_TYPE_CONTINUOUS   0
#define VIP_ACQ_TYPE_ACCUMULATION 1
#define VIP_WL_MAPPING_LINEAR     0
#define VIP_WL_MAPPING_NORM_ATAN  1
#define VIP_WL_MAPPING_CUSTOM     2

// Mode Acquisition Type Constants.
#define VIP_INVALID_ACQ_MODE_TYPE  -1
#define VIP_VALID_XRAYS_N_FRAMES   0
#define VIP_VALID_XRAYS_ALL_FRAMES 1 // Defined for future implementation.
#define VIP_AUTO_SENSE_N_FRAMES    2
#define VIP_AUTO_SENSE_ALL_FRAMES  3

// Opcode.
#define VIP_AUTO_GAIN_CTRL     0x12
#define VIP_AUTO_GAIN_CTRL_ON  1
#define VIP_AUTO_GAIN_CTRL_OFF 0

#define VIP_VERT_REVERSAL_OP   0x1d
#define VIP_VERT_REVERSAL_ON   1
#define VIP_VERT_REVERSAL_OFF  0

#define VIP_HORZ_REVERSAL_OP   0x1e
#define VIP_HORZ_REVERSAL_ON   1
#define VIP_HORZ_REVERSAL_OFF  0

// for Viva use only: viv_header definition; vip_comm does not define VIV_HEADER_INC
#if defined(VIV_HEADER_INC)
#include "viv_hdr.h"
#endif // VIV_HEADER_INC

// DLL interface functions.
extern "C" LINKAGE int vip_analog_offset_cal(int mode_num); // 3.
extern "C" LINKAGE int vip_check_link(void); // 4.
extern "C" LINKAGE int vip_close_link(BOOL unregister = FALSE); // 5.
extern "C" LINKAGE int vip_enable_auto_cal(int mode_num,
                                           BOOL enable,
                                           int minimum_delay,
                                           int post_exposure_delay); // 6.
extern "C" LINKAGE int vip_enable_sw_handshaking(BOOL enable); // 7.
extern "C" LINKAGE int vip_gain_cal_prepare(int mode_num, BOOL auto_sense = FALSE); // 8.
extern "C" LINKAGE int vip_get_analog_offset_data(int modeNum, 
                                                  int *targetVal,
                                                  int *tolerance,
                                                  int *medianPercent,
                                                  double *fractionalIterationVal,
                                                  int *numberOfIterations); // 9.
extern "C" LINKAGE int vip_get_analog_offset_stats(int mode_mum,
                                                   int num_asics,
                                                   double *standard_deviation,
                                                   int *segment_defect_found); // 10.
extern "C" LINKAGE int vip_get_cal_stats(int mode_num,
                                         int *gain_median,
                                         double *gain_sigma,
                                         int *offset_median); // 11.
extern "C" LINKAGE int vip_get_config_data(char *full_file_path,
                                           char *target_file_name); // 12.
extern "C" LINKAGE int vip_get_correction(BOOL *offset_cal,
                                          BOOL *gain_cal,
                                          BOOL *defect_map,
                                          BOOL *line_noise); // 13.
extern "C" LINKAGE int vip_get_current_mode(int *mode_num); // 14.
extern "C" LINKAGE int vip_get_image(int mode_num,
                                     int image_type,
                                     int x_size,
                                     int y_size,
                                     WORD *image_ptr); // 15.
extern "C" LINKAGE int vip_get_lih(int mode_num,
                                   BOOL *lih_active); // 16.
extern "C" LINKAGE int vip_get_mode_acq_type(int mode_num,
                                             int *mode_acq_type,
                                             int *num_frames); // 17.
extern "C" LINKAGE int vip_get_mode_details(int mode_num,
                                            int *acq_type,
                                            double *frame_rate,
                                            double *analog_gain,
                                            int *lines_per_frame,
                                            int *columns_per_frame,
                                            int *lines_per_pixel,
                                            int *columns_per_pixel,
                                            char *mode_description,
											int *dcds_enable = NULL); // 18.
extern "C" LINKAGE int vip_get_num_acq_frames(int mode_num,
                                              int *num_acq_frames); // 19.
extern "C" LINKAGE int vip_get_num_cal_frames(int mode_num,
                                              int *num_cal_frames); // 20.
extern "C" LINKAGE int vip_get_rad_scaling(int mode_num,
                                           int *scaling_type,
                                           int *target_value); // 21.
extern "C" LINKAGE int vip_get_recursive_filter(int mode_num,
                                                double *buffer_weight); // 22.
extern "C" LINKAGE int vip_get_self_test_log(void); // 23.
extern "C" LINKAGE int vip_get_system_info(int *num_modes,
                                           int *default_mode_num,
                                           int *max_lines_per_frame,
                                           int *max_columns_per_frame,
                                           int *max_pixel_value,
                                           BOOL *has_video,
                                           char *system_description,
                                           int *startup_configuration = NULL,
                                           int *num_asics = NULL,
										   int *receptor_type = NULL); // 24.
extern "C" LINKAGE int vip_get_system_version_numbers(int sys_ver_type,
                                                      char *ver_str); // 25.
extern "C" LINKAGE int vip_get_wl(int *bot,
                                  int *top,
                                  int *mapping); // 26.
extern "C" LINKAGE int vip_offset_cal(int mode_num); // 28.
extern "C" LINKAGE int vip_open_link(int new_link = VIP_ETHERNET_LINK,
                                     const char *client_ip_address = NULL,
                                     const char *server_ip_address = NULL,
                                     int com_port = VIP_COM1); // 29.
extern "C" LINKAGE int vip_put_config_data(char *full_file_path,
                                           char *target_file_name); // 30.
extern "C" LINKAGE int vip_put_image(int mode_num,
                                     int image_type,
                                     int x_size,
                                     int y_size,
                                     WORD *image_ptr); // 31.
extern "C" LINKAGE int vip_query_error(int *error_mask); // 32.
extern "C" LINKAGE int vip_query_progress(int *num_frames,
                                          BOOL *complete,
                                          int *num_pulses = NULL,
                                          BOOL *ready_for_pulse = NULL); // 33.
extern "C" LINKAGE int vip_register_application(const char *client_ip_address = NULL,
                                                const char *server_ip_addr = NULL); // 34.
extern "C" LINKAGE int vip_reset_state(void); // 35.
extern "C" LINKAGE int vip_select_mode(int mode_num); // 36.
extern "C" LINKAGE int vip_self_test(void); // 37.
extern "C" LINKAGE int vip_set_analog_offset_data(int mode_num, 
                                                  int target_value,
                                                  int tolerance,
                                                  int median_percent,
                                                  double fractional_iteration_delta,
                                                  int number_iterations); // 39.
extern "C" LINKAGE int vip_set_correction(BOOL offset_cal,
                                          BOOL gain_cal,
                                          BOOL defect_map,
                                          BOOL line_noise); // 40.
extern "C" LINKAGE int vip_set_debug(BOOL enable); // 41.
extern "C" LINKAGE int vip_dcds_enable(BOOL enable); 
extern "C" LINKAGE int vip_set_frame_rate(int mode_num,
                                          double frame_rate);
extern "C" LINKAGE int vip_set_lih(int mode_num,
                                   BOOL lih_active); // 43.
extern "C" LINKAGE int vip_set_mode_acq_type(int mode_num,
                                             int acq_type,
                                             int num_frames); // 44.
extern "C" LINKAGE int vip_set_num_acq_frames(int mode_num,
                                              int num_acq_frames); // 45.
extern "C" LINKAGE int vip_set_num_cal_frames(int mode_num,
                                              int num_cal_frames); // 46.
extern "C" LINKAGE int vip_set_rad_scaling(int mode_num,
                                           int scaling_type,
                                           int target_value); // 47.
extern "C" LINKAGE int vip_set_recursive_filter(int mode_num,
                                                double buffer_weight); // 48.
extern "C" LINKAGE int vip_set_wl(int bot,
                                  int top,
                                  int mapping); // 49.
extern "C" LINKAGE int vip_sw_handshaking(int signal_type,
                                          BOOL active); // 50.















// These commands are not intended for public use!
// Specials for ABS board.
extern "C" LINKAGE int vip_acc_board_peek(unsigned long address,
                                          unsigned long *value); // 1.
extern "C" LINKAGE int vip_acc_board_poke(unsigned long address,
                                          unsigned long value); // 2.
extern "C" LINKAGE int vip_initialize_abs(int num_ints,
                                          int delay,
                                          int store_addr,
                                          int wait_for_status,
                                          int send_go,
                                          int connect_isr); // 27.
extern "C" LINKAGE int vip_set_abs_delay(int delay); // 38.
extern "C" LINKAGE int vip_write_abs_mailbox(UINT opcode,
                                             UINT data); // 51.
extern "C" LINKAGE int vip_write_mult_cmds(int numCmds, UINT *cmdArray); // 52.

#endif // VIP_HEADER